@@ -1,12 +1,18 @@
+#coding=utf-8
+
+from copy import copy
 from datetime import datetime
-import time
-import sys
 import getpass
 import json
+import mimetypes
 import pickle
 from pprint import pprint
-from urllib.parse import urlencode
-import mimetypes
+import sys
+import time
+try: # python3
+    from urllib.parse import urlencode
+except ImportError: # python2
+    from urllib import urlencode
 
 import httplib2
 
@@ -15,14 +21,18 @@ SERVER_ENCODING = 'utf-8'
 LIST_LIMIT = 400
 
 httplib2.debuglevel = 4 if DEBUG else 0
-http = httplib2.Http('.cache')
-
 
 class WikiError(Exception):
     def __init__(self, code, msg):
         super(WikiError, self).__init__('WikiError: %s' % msg)
         self.code = code
 
+class ChainedRedirectError(Exception):
+    def __init__(self, page):
+        super(ChainedRedirectError, self).__init__(
+            'Too much redirects on page %s' % page.title
+        )
+
 
 def utf8(x):
     if not x:
@@ -78,16 +88,33 @@ class Wiki():
     namespaces_names = None # map from id to localised name
     namespaces_ids = None # map from canonical name to id
 
-    def __init__(self, endpoint=None, throttle=1):
+    def __init__(self, endpoint=None, throttle=1,
+        disable_ssl_certificate_validation=False
+    ):
         ''' Create wiki client for given endpoint addres.  '''
         self._tokens = {}
         self.endpoint = endpoint
         self.throttle = throttle
 
+
+        self.http = httplib2.Http('.cache')
+        self.http.disable_ssl_certificate_validation = disable_ssl_certificate_validation
+
+
+
         if endpoint:
             self.get_namespaces()
         # else may be load session
 
+    def get_unified_login_to(self, domain):
+        ''' Returns new wiki instance with changed domain and namespace '''
+        res = copy(self)
+        url = self.endpoint.split('/') # http://domain/...
+        url[2] = domain
+        res.endpoint = '/'.join(url)
+        res.get_namespaces()
+        return res
+
     def load(self, filename):
         ''' Load cookies, endpoint, namespaces. '''
         try:
@@ -127,6 +154,17 @@ class Wiki():
         assert self.endpoint, "Can't make a request without endpoint"
         if params is None:
             params = {}
+
+        p2 = {}
+        for k,v in params.items():
+            if v is True:
+                p2[k] = 1
+            elif v is False:
+                pass
+            else:
+                p2[k] = v
+        params = p2
+
         params['format'] = 'json'
         if headers is None:
             headers = {}
@@ -151,7 +189,7 @@ class Wiki():
                 )
                 headers['Content-type'] = content_type
         time.sleep(self.throttle)
-        responce, content = http.request(
+        responce, content = self.http.request(
             query,
             method,
             headers=headers,
@@ -236,17 +274,14 @@ class Wiki():
         else:
             raise WikiError(content['result'], 'Login error: %s.' % content['result'])
 
-    def set_endpoint(self, url):
-        self.endpoint = 'http://%s/w/api.php' % url
-
     def session_login(self, endpoint=None, login=None, password=None):
         ''' Start session for given account. '''
         if self.endpoint:
-            self.set_endpoint(endpoint)
+            self.endpoint = endpoint
         else:
             if not endpoint:
                 endpoint = input('Endpoint: ')
-            self.set_endpoint(endpoint)
+            self.endpoint = endpoint
 
         if not login:
             login = input('Login: ')
@@ -525,7 +560,7 @@ class Page:
             self.read()
         return self.namespace == self.wiki.namespaces_ids['Category']
 
-    def write(self, text, summary):
+    def write(self, text, summary, bot=True, minor=False):
         ''' Replace page contents with given text. 
         summary param is the description of edit.
         '''
@@ -536,6 +571,8 @@ class Page:
                 text=text,
                 token=self.wiki.get_token('edit', titles=self.title),
                 summary=summary,
+                bot=bot,
+                minor=minor,
             ),
             method='POST',
         )
@@ -623,9 +660,31 @@ class Page:
             res[link['lang']] = link['*']
         return res
 
+    def categories(self):
+        ''' Return list of categories for page '''
+        q = {
+            'prop': 'categories',
+            'cllimit': LIST_LIMIT,
+            'titles': self.title,
+        }
+        res, cont = self.wiki.query_cont(q)
+        res = res['pages'].popitem()[1]
+        if 'categories' not in res:
+            return
+        for cat in res['categories']:
+            if 'title' in cat:
+                yield self.wiki.category(cat['title'])
+        while cont:
+            q.update(cont['categories'])
+            res, cont = self.wiki.query_cont(q)
+            for cat in res['categories']:
+                if 'title' in cat:
+                    yield self.wiki.category(cat['title'])
+
     def redirect(self):
         ''' Return None or page to which it redirects
         '''
+        self.read() # Normalise title, otherwise sometimes don't work
         redirects = self.wiki.query({
             'titles': self.title,
             'redirects': None,
@@ -634,6 +693,18 @@ class Page:
             if r['from'] == self.title:
                 return self.wiki.page(r['to'])
 
+    def resolve_redirect(self):
+        ''' Return page or page to which this redirects if it is redirect '''
+        res = self
+        i = 0
+        while res.redirect():
+            res = res.redirect()
+            i += 1
+            if i > 3:
+                raise ChainedRedirectError(self)
+        return res
+
+
     def protect(self, reason):
         req = dict(
             action='protect',
@@ -655,12 +726,14 @@ class Category(Page):
             title = wiki.CATEGORY_NS_NAME + ':' + title
         super(Category, self).__init__(wiki, title)
 
-    def members(self):
+    def members(self, namespace=None):
         q = {
             'list': 'categorymembers',
             'cmlimit': LIST_LIMIT,
             'cmtitle': self.title,
         }
+        if namespace is not None:
+            q['cmnamespace'] = namespace
         res, cont = self.wiki.query_cont(q)
         for member in res['categorymembers']:
             yield self.wiki.page(member['title'])
@@ -711,11 +784,14 @@ class File(Page):
         if width:
             query['iiurlwidth'] = width
         resp = self.wiki.query(query)
-        info = list(resp['pages'].values())[0]['imageinfo'][0]
+        try:
+            info = list(resp['pages'].values())[0]['imageinfo'][0]
+        except KeyError:
+            print(resp)
         return info['thumburl' if width else 'url']
 
     def download(self, to_file, width=None):
-        headers, content = http.request(self._url(width))
+        headers, content = self.wiki.http.request(self._url(width))
         with open(to_file, 'wb') as f:
             f.write(content)
 
