@@ -1,8 +1,25 @@
+#coding=utf-8
 '''
 User interface utilities
 '''
 #from mako.template import Template
 
+colors = {
+    '-': 91,
+    '+': 92,
+    '?': 94,
+    'red': 91,
+    'green': 92,
+    'blue': 94,
+}
+
+def cprint(*args, **kwargs):
+    color = colors.get(kwargs.get('color'), 0)
+    if color:
+        print('\033[%sm' % color, *(args + ('\033[0m',)))
+    else:
+        print(*args)
+
 def print_diff(text_a, text_b):
     ''' Print difference between two values '''
     a_lines = text_a.splitlines(1)
@@ -10,11 +27,7 @@ def print_diff(text_a, text_b):
     diff = ndiff(a_lines, b_lines)
 
     for line in diff:
-        print('\033[%sm%s\033[0m' % ({
-            '-':91,
-            '+':92,
-            '?':94,
-        }.get(line[0], 0), line[:-1]))
+        cprint(line[:-1], color=line[0])
 
 def ask_y_n(question):
     while True:
